@@ -6,25 +6,29 @@ import functools
 
 from contextlib import contextmanager
 from pprint import pformat
-from json.decoder import JSONDecodeError
 from uuid import uuid4
 
 from functools import partial
 from ..errors import SpintopException
 
 from ..auth import AuthModule
-from ..services.auth0 import Auth0Provider
-
 from ..logs import _logger
 from ..compat import VERSION
 from ..storage import TEMP_DIR
+
 from .tests_facade import SpintopAPIPersistenceFacade
+from .analytics import SpintopAPIAnalytics
+
+from spintop.utils.requests_utils import spintop_session
 
 logger = _logger('spintop-api')
 
 TEMPORARY_DOWNLOAD_FOLDER = os.path.join(TEMP_DIR, '.spintop-api-download-cache')
 if not os.path.exists(TEMPORARY_DOWNLOAD_FOLDER):
-    os.makedirs(TEMPORARY_DOWNLOAD_FOLDER)
+    try:
+        os.makedirs(TEMPORARY_DOWNLOAD_FOLDER)
+    except:
+        logger.warning(f'Unable to makedirs({TEMPORARY_DOWNLOAD_FOLDER}) even though file does not exist.')
     
 class SpintopClientError(SpintopException):
     pass
@@ -33,7 +37,7 @@ class SpintopAPIError(SpintopException):
     def __init__(self, resp):
         try:
             json = resp.json()
-        except JSONDecodeError:
+        except Exception:
             json = {}
         
         error_type = json.get('error_type', 'UNKNOWN')
@@ -80,7 +84,7 @@ def SpintopAPISession(prefix, auth_module):
         try:
             resp = attempt_request(*args, **kwargs)
         except SpintopAPIError as e:
-            if e.resp.status_code == 401 and e.error_type in ('auth_token_verification_failed',) :
+            if e.resp.status_code == 401 and e.error_type in ('auth_token_verification_failed', 'auth_missing_header') :
                 # Attempt ONCE to refresh the credentials and retry.
                 auth_module.refresh_credentials()
                 resp = attempt_request(*args, **kwargs)
@@ -88,22 +92,9 @@ def SpintopAPISession(prefix, auth_module):
                 raise
         return resp
     
-    s = requests.Session()                                                                                                                                                                                                                                                                                                                     
-    s.request = partial(new_request, prefix, s.request)                                                                                                                                                                                                                                                                                            
-    return s  
-
-class NoAuthModule(object):
-    def get_auth_headers(self):
-        return {}
-
-    def refresh_credentials(self):
-        raise NotImplementedError('Cannot refresh credentials without an auth module.')
-
-class SpintopAPIAuthProvider(Auth0Provider):
-    @functools.lru_cache(maxsize=128)
-    def get_user_orgs(self, access_token):
-        token_payload = self.jwt_verify(access_token)
-        return token_payload.get('https://spinhub.ca/authorization', {}).get('groups', [])
+    s = spintop_session()
+    s.request = partial(new_request, prefix, s.request)
+    return s
 
 class SpintopAPIClientModule(object):
     def __init__(self, api_url, selected_org_id=None, auth=None):
@@ -112,20 +103,14 @@ class SpintopAPIClientModule(object):
         self.auth = auth
         
         self._selected_org_id = selected_org_id
-        self.tests_facade = SpintopAPIPersistenceFacade(self)
+        self._tests_facade = SpintopAPIPersistenceFacade(self)
+        self._analytics = SpintopAPIAnalytics(self)
         
         self._api_url = api_url
         self._user_info = None
         self._api_spec = None
         self._default_org = None
         self._machine = None
-    
-    @classmethod
-    def init(cls, config, auth):
-        return cls(
-            api_url=lambda: config.get_selected_profile()['spintop_api_url'],
-            auth=auth
-        )
 
     @property
     def api_url(self):
@@ -170,7 +155,37 @@ class SpintopAPIClientModule(object):
 
     @property
     def tests(self):
-        return self.tests_facade
+        return self._tests_facade
+
+    @property
+    def analytics(self):
+        return self._analytics
+
+    def login(self):
+        """Interactive login"""
+        self.auth.assert_no_login() # Check first before prompting
+
+        with self.login_browser() as device_flow_response:
+            logger.info(f'User code is {device_flow_response.user_code!r}')
+            logger.info(f'If this device is headless (without a browser), you may visit this link to authorize this device on another machine: {device_flow_response.verification_uri_complete}')
+
+    def login_user_pass(self, username, password):
+        self.auth.login_user_pass(username, password)
+
+    def login_browser(self):
+        return self.auth.login_device_flow()
+
+    def stored_logged_username(self):
+        if self.auth.credentials:
+            return self.auth.credentials.get('username')
+        else:
+            return None
+    
+    def logout(self):
+        return self.auth.logout()
+    
+    def get_user_orgs(self):
+        return self.auth.user_orgs
 
     def new_session(self):
         return SpintopAPISession(self.api_url, self.auth)
@@ -182,8 +197,22 @@ class SpintopAPIClientModule(object):
         resp = self.session.get(self.get_link('core.private'))
         return resp
 
-    def get_org_info(self, org_name):
-        return self.session.get(self.get_link('tenancy.org_info', org_id=org_name)).json()
+    def get_org_info(self, org_id=None):
+        return self.session.get(self.get_link('tenancy.org_info', org_id=org_id)).json()
+
+    def create_org(self, org_id):
+        return self.session.post(self.get_link('tenancy.create_org', org_id=org_id)).json()
+
+    def get_tests_env(self, org_id=None):
+        """ An env that points to the API itself """
+        return self.session.get(self.get_link('tests.env', org_id=org_id)).json()
+
+    def get_tests_env_direct_database_context(self, org_id=None):
+        """ An env that points to the API's underlying database (with the API's URI)"""
+        if org_id is None: org_id = self.selected_org_id
+        env = self.get_tests_env(org_id)
+        org_info = self.get_org_info(org_id)
+        return env.new_database_context(org_info['database_name'])
 
     def get_link(self, link_name, org_id=None, **kwargs):
         links = self.api_spec['_links']
