@@ -1,9 +1,12 @@
 import inspect
 from reprlib import recursive_repr
-from collections import Mapping
+from datetime import datetime
+from collections.abc import Mapping
 
 from dataclasses import dataclass, fields, _MISSING_TYPE, MISSING, field, asdict
 
+from spintop.utils import GetRecursiveMixin, GetRecursive
+
 from .view import DataClassPrimitiveView, join_key_prefix
 from .meta import MetaBaseDataClass, MetaField
 
@@ -17,6 +20,22 @@ def register_type(_type):
         return cls
     return _wrap_cls
 
+def cls_of_obj(obj):
+    try:
+        _type = type_of(obj)
+        cls = cls_of_type(_type)
+        if cls is None:
+            raise AttributeError()
+        return cls
+    except AttributeError:
+        return cls_of_serialized(obj)
+
+def cls_of_serialized(obj_serialized):
+    return cls_of_type(serialized_type_of(obj_serialized))
+
+def cls_of_type(_type):
+    return _TYPE_MAP.get(_type, None)
+
 def type_of(cls):
     return cls._type
 
@@ -63,11 +82,23 @@ _fields_cache = FieldsCache()
 _default_views_cache = DefaultViewsCache()
 
 def model_dataclass(**kwargs):
-    return dataclass(init=False, repr=False, **kwargs)
+    """Custom dataclass decorator that removes the default values set on classes.
+    This allows the meta base data class to return the field itself instead of the
+    default value."""
+    def _decorator(cls):
+        cls = dataclass(init=False, repr=False, **kwargs)(cls)
+        fields = cls.get_fields()
+        for field in fields:
+            try:
+                delattr(cls, field.name)
+            except AttributeError:
+                pass
+        return cls
+    return _decorator
 
 @model_dataclass()
 @register_type('_base')
-class BaseDataClass(metaclass=MetaBaseDataClass):
+class BaseDataClass(GetRecursiveMixin, metaclass=MetaBaseDataClass):
     
     def __init__(self, *args, **kwargs):
         if args:
@@ -76,6 +107,10 @@ class BaseDataClass(metaclass=MetaBaseDataClass):
 
         self.update_attrs(defaults) # First set defaults
         self.update_attrs(kwargs) # Then constructor arguments.
+        self.__post_init__()
+
+    def __post_init__(self):
+        pass
 
     @classmethod
     def defaults(cls, **others):
@@ -155,40 +190,36 @@ class BaseDataClass(metaclass=MetaBaseDataClass):
     def cls_data_from_dict(cls, data):
         _type = data.pop('_type', None)
         if _type:
-            cls = _TYPE_MAP[_type]
+            cls = cls_of_type(_type)
         return cls, data
-    
-    def get_recursive(self, as_str=None, as_array=None):
-        if as_str:
-            as_array = as_str.split('.')
-        
-        this_attr_name, *as_array = as_array
-        this_attr = getattr(self, this_attr_name)
-        if as_array:
-            return this_attr.get_recursive(as_array=as_array)
-        else:
-            return this_attr
 
     def getattr_from_field(self, meta_field):
-        if isinstance(meta_field, property):
-            return meta_field.__get__(self, type(self))
-        elif meta_field.is_container_of_class(self.__class__):
-            return self.get_recursive(as_array=meta_field.name_tuple_)
-        else:
-            raise AttributeError('This class {!r} is not a subclass of the meta field type {!r}'.format(self.__class__, meta_field.container_type_))
+        return getattr_from_field(self, meta_field)
 
     def hasattr_from_field(self, meta_field):
-        try:
-            self.getattr_from_field(meta_field)
-            return True
-        except AttributeError:
-            return False
+        return hasattr_from_field(self, meta_field)
 
     @recursive_repr()
     def __repr__(self):
         fields = tuple("{}={}".format(field.name, repr(getattr(self, field.name, None))) for field in self.get_fields())
         return "{}({})".format(self.__class__.__name__, ", ".join(fields))
 
+def getattr_from_field(obj, meta_field):
+    obj_cls = cls_of_obj(obj)
+    if isinstance(meta_field, property):
+        return meta_field.__get__(obj, type(obj))
+    elif meta_field.is_container_of_cls(obj_cls):
+        return GetRecursive(obj).get_recursive(as_array=meta_field.name_tuple_)
+    else:
+        raise AttributeError('This class {!r} is not a subclass of the meta field type {!r}'.format(obj_cls, meta_field.container_type_))
+
+def hasattr_from_field(obj, meta_field):
+    try:
+        getattr_from_field(obj, meta_field)
+        return True
+    except AttributeError:
+        return False
+
 
 def copy_nullify_fields(obj, null_fields, target_cls=None):
     if target_cls is None:
@@ -206,6 +237,11 @@ def default_field_value(field):
         value = field.type.null()
     elif not isinstance(field.default_factory, _MISSING_TYPE):
         value = field.default_factory()
+    elif not isinstance(field.default, _MISSING_TYPE):
+        if callable(field.default):
+            value = field.default()
+        else:
+            value = field.default
     else:
         value = None
     
